@@ -48,6 +48,30 @@
 ;; When haskell-align-imports is run within the same buffer, the
 ;; import list is transformed to:
 ;;
+;; import "abc"            Eight
+;; import qualified        Eighteen as PRELUDE hiding (A)
+;; import qualified "defg" Eleven as PRELUDE
+;; import                  Fifteen hiding (A)
+;; import                  Five (A)
+;; import qualified        Four as PRELUDE
+;; import qualified "z"    Fourteen  (A,B)
+;; import "abc"            Nine as TWO
+;; import "abc"            Nineteen hiding (A)
+;; import                  One
+;; import qualified        Seven (A,B)
+;; import qualified        Seventeen hiding (A)
+;; import                  Six (A,B)
+;; import                  Sixteen as TWO hiding (A)
+;; import qualified "abc"  Ten
+;; import "zotconpop"      Thirteen (A,B)
+;; import qualified        Three
+;; import "barmu"          Twelve (A)
+;; import "abc"            Twenty as TWO hiding (A)
+;; import                  Two as A
+;;
+;; If you want everything after module names to be padded out, too,
+;; customize `haskell-align-imports-pad-after-name', and you'll get:
+;;
 ;; import                  One
 ;; import                  Two       as A
 ;; import qualified        Three
@@ -71,13 +95,24 @@
 
 ;;; Code:
 
-(with-no-warnings (require 'cl))
+(require 'cl-lib)
 
 (defvar haskell-align-imports-regexp
   (concat "^\\(import[ ]+\\)"
           "\\(qualified \\)?"
           "[ ]*\\(\"[^\"]*\" \\)?"
-          "[ ]*\\([A-Za-z0-9_.']*.*\\)"))
+          "[ ]*\\([A-Za-z0-9_.']+\\)"
+          "[ ]*\\([ ]*as [A-Z][^ ]*\\)?"
+          "[ ]*\\((.*)\\)?"
+          "\\([ ]*hiding (.*)\\)?"
+          "\\( -- .*\\)?[ ]*$")
+  "Regex used for matching components of an import.")
+
+(defcustom haskell-align-imports-pad-after-name
+  nil
+  "Pad layout after the module name also."
+  :type 'boolean
+  :group 'haskell-interactive)
 
 ;;;###autoload
 (defun haskell-align-imports ()
@@ -112,8 +147,8 @@
         (when line
           (let ((match
                  (haskell-align-imports-merge-parts
-                  (loop for i from 1 to 8
-                        collect (haskell-align-imports-chomp (match-string i line))))))
+                  (cl-loop for i from 1 to 8
+                           collect (haskell-align-imports-chomp (match-string i line))))))
             (setq imports (cons (cons match (line-beginning-position))
                                 imports)))))
       (forward-line))
@@ -123,22 +158,31 @@
   "Merge together parts of an import statement that shouldn't be separated."
   (let ((parts (apply #'vector l))
         (join (lambda (ls)
-                (reduce (lambda (a b)
-                          (concat a
-                                  (if (and (> (length a) 0)
-                                           (> (length b) 0))
-                                      " "
-                                    "")
-                                  b))
-                        ls))))
-    (list (funcall join (list (aref parts 0)
-                              (aref parts 1)
-                              (aref parts 2)))
-          (aref parts 3)
-          (funcall join (list (aref parts 4)
-                              (aref parts 5)
-                              (aref parts 6)))
-          (aref parts 7))))
+                (cl-reduce (lambda (a b)
+                             (concat a
+                                     (if (and (> (length a) 0)
+                                              (> (length b) 0))
+                                         " "
+                                       "")
+                                     b))
+                           ls))))
+    (if haskell-align-imports-pad-after-name
+        (list (funcall join (list (aref parts 0)
+                                  (aref parts 1)
+                                  (aref parts 2)))
+              (aref parts 3)
+              (funcall join (list (aref parts 4)
+                                  (aref parts 5)
+                                  (aref parts 6)))
+              (aref parts 7))
+      (list (funcall join (list (aref parts 0)
+                                (aref parts 1)
+                                (aref parts 2)))
+            (funcall join (list (aref parts 3)
+                                (aref parts 4)
+                                (aref parts 5)
+                                (aref parts 6)
+                                (aref parts 7)))))))
 
 (defun haskell-align-imports-chomp (str)
   "Chomp leading and tailing whitespace from STR."
@@ -149,19 +193,21 @@
 
 (defun haskell-align-imports-padding (imports)
   "Find the padding for each part of the import statements."
-  (reduce (lambda (a b) (mapcar* #'max a b))
-          (mapcar (lambda (x) (mapcar #'length (car x)))
-                  imports)))
+  (if (null imports)
+      imports
+    (cl-reduce (lambda (a b) (cl-mapcar #'max a b))
+               (mapcar (lambda (x) (mapcar #'length (car x)))
+                       imports))))
 
 (defun haskell-align-imports-fill (padding line)
   "Fill an import line using the padding worked out from all statements."
   (mapconcat #'identity
-             (mapcar* (lambda (pad part)
-                        (if (> (length part) 0)
-                            (concat part (make-string (- pad (length part)) ? ))
-                          (make-string pad ? )))
-                      padding
-                      line)
+             (cl-mapcar (lambda (pad part)
+                          (if (> (length part) 0)
+                              (concat part (make-string (- pad (length part)) ? ))
+                            (make-string pad ? )))
+                        padding
+                        line)
              " "))
 
 (defun haskell-align-imports-line-match-it ()
@@ -176,13 +222,9 @@
   "Are we after the imports list?"
   (save-excursion
     (goto-char (line-beginning-position))
-    (not (not (search-forward-regexp "\\( = \\|\\<instance\\>\\| :: \\)"
+    (not (not (search-forward-regexp "\\( = \\|\\<instance\\>\\| :: \\| ∷ \\)"
                                      (line-end-position) t 1)))))
 
 (provide 'haskell-align-imports)
 
-;; Local Variables:
-;; byte-compile-warnings: (not cl-functions)
-;; End:
-
 ;;; haskell-align-imports.el ends here
