@@ -131,7 +131,6 @@
     (define-key map (kbd "M-g b") 'haskell-cabal-goto-benchmark-section)
     (define-key map (kbd "M-g t") 'haskell-cabal-goto-test-suite-section)
     map))
-(defvar haskell-cabal-mode-map (make-sparse-keymap))
 
 ;;;###autoload
 (define-derived-mode haskell-cabal-mode fundamental-mode "Haskell-Cabal"
@@ -173,14 +172,14 @@
           val)))))
 
 ;;;###autoload
-(defun haskell-guess-setting (name)
+(defun haskell-cabal-guess-setting (name)
   "Guess the specified setting of this project.
 If there is no valid .cabal file to get the setting from (or
 there is no corresponding setting with that name in the .cabal
 file), then this function returns nil."
   (interactive)
   (when (and name buffer-file-name)
-    (let ((cabal-file (haskell-cabal-find-file (file-name-directory buffer-file-name))))
+    (let ((cabal-file (haskell-cabal-find-file)))
       (when (and cabal-file (file-readable-p cabal-file))
         (with-temp-buffer
           (insert-file-contents cabal-file)
@@ -211,21 +210,25 @@ Return nil if no Cabal description file could be located via
 If DIR is nil, `default-directory' is used as starting point for
 directory traversal.  Upward traversal is aborted if file owner
 changes.  Uses`haskell-cabal-find-pkg-desc' internally."
-  (catch 'found
-    (let ((user (nth 2 (file-attributes (or dir default-directory))))
-          ;; Abbreviate, so as to stop when we cross ~/.
-          (root (abbreviate-file-name (or dir default-directory))))
-      ;; traverse current dir up to root as long as file owner doesn't change
-      (while (and root (equal user (nth 2 (file-attributes root))))
-        (let ((cabal-file (haskell-cabal-find-pkg-desc root)))
-          (when cabal-file
-            (throw 'found cabal-file)))
+  (let ((use-dir (or dir default-directory)))
+    (while (and use-dir (not (file-directory-p use-dir)))
+      (setq use-dir (file-name-directory (directory-file-name use-dir))))
+    (when use-dir
+      (catch 'found
+        (let ((user (nth 2 (file-attributes use-dir)))
+              ;; Abbreviate, so as to stop when we cross ~/.
+              (root (abbreviate-file-name use-dir)))
+          ;; traverse current dir up to root as long as file owner doesn't change
+          (while (and root (equal user (nth 2 (file-attributes root))))
+            (let ((cabal-file (haskell-cabal-find-pkg-desc root)))
+              (when cabal-file
+                (throw 'found cabal-file)))
 
-        (let ((proot (file-name-directory (directory-file-name root))))
-          (if (equal proot root) ;; fix-point reached?
-              (throw 'found nil)
-            (setq root proot))))
-      nil)))
+            (let ((proot (file-name-directory (directory-file-name root))))
+              (if (equal proot root) ;; fix-point reached?
+                  (throw 'found nil)
+                (setq root proot))))
+          nil)))))
 
 (defun haskell-cabal-find-pkg-desc (dir &optional allow-multiple)
   "Find a package description file in the directory DIR.
@@ -295,7 +298,8 @@ OTHER-WINDOW use `find-file-other-window'."
     "hscolour"
     "register"
     "test"
-    "help"))
+    "help"
+    "run"))
 
 
 (defgroup haskell-cabal nil
