@@ -1,5 +1,11 @@
 ;;; haskell-commands.el --- Commands that can be run on the process
 
+;;; Commentary:
+
+;;; This module provides varoius `haskell-mode' and `haskell-interactive-mode'
+;;; specific commands such as show type signature, show info, haskell process
+;;; commands and etc.
+
 ;; Copyright (c) 2014 Chris Done. All rights reserved.
 
 ;; This file is free software; you can redistribute it and/or modify
@@ -24,8 +30,15 @@
 (require 'haskell-font-lock)
 (require 'haskell-interactive-mode)
 (require 'haskell-session)
+(require 'haskell-presentation-mode)
 (require 'highlight-uses-mode)
 
+
+(defvar haskell-utils-async-post-command-flag nil
+  "Non-nil means some commands were triggered during async function execution.")
+(make-variable-buffer-local 'haskell-utils-async-post-command-flag)
+
+
 ;;;###autoload
 (defun haskell-process-restart ()
   "Restart the inferior Haskell process."
@@ -35,7 +48,8 @@
   (haskell-process-start (haskell-interactive-session)))
 
 (defun haskell-process-start (session)
-  "Start the inferior Haskell process."
+  "Start the inferior Haskell process with a given SESSION.
+You can create new session using function `haskell-session-make'."
   (let ((existing-process (get-process (haskell-session-name (haskell-interactive-session)))))
     (when (processp existing-process)
       (haskell-interactive-mode-echo session "Restarting process ...")
@@ -71,7 +85,7 @@
     process))
 
 (defun haskell-process-send-startup (process)
-  "Send the necessary start messages."
+  "Send the necessary start messages to haskell PROCESS."
   (haskell-process-queue-command
    process
    (make-haskell-command
@@ -130,7 +144,9 @@ If I break, you can:
   (interrupt-process (haskell-process-process (haskell-commands-process))))
 
 (defun haskell-process-reload-with-fbytecode (process module-buffer)
-  "Reload FILE-NAME with -fbyte-code set, and then restore -fobject-code."
+  "Query a PROCESS to reload MODULE-BUFFER with -fbyte-code set.
+Restores -fobject-code after reload finished.
+MODULE-BUFFER is the actual Emacs buffer of the module being loaded."
   (haskell-process-queue-without-filters process ":set -fbyte-code")
   (haskell-process-touch-buffer process module-buffer)
   (haskell-process-queue-without-filters process ":reload")
@@ -138,8 +154,8 @@ If I break, you can:
 
 ;;;###autoload
 (defun haskell-process-touch-buffer (process buffer)
-  "Updates mtime on the file for BUFFER by queing a touch on
-PROCESS."
+  "Query PROCESS to `:!touch` BUFFER's file.
+Use to update mtime on BUFFER's file."
   (interactive)
   (haskell-process-queue-command
    process
@@ -160,7 +176,8 @@ PROCESS."
 (defvar url-http-end-of-headers)
 
 (defun haskell-process-hayoo-ident (ident)
-  "Hayoo for IDENT, returns a list of modules asyncronously through CALLBACK."
+  ;; FIXME Obsolete doc string, CALLBACK is not used.
+  "Hayoo for IDENT, return a list of modules asyncronously through CALLBACK."
   ;; We need a real/simulated closure, because otherwise these
   ;; variables will be unbound when the url-retrieve callback is
   ;; called.
@@ -183,7 +200,7 @@ PROCESS."
         (warn "HTTP error %s fetching %s" url-http-response-status url)))))
 
 (defun haskell-process-hoogle-ident (ident)
-  "Hoogle for IDENT, returns a list of modules."
+  "Hoogle for IDENT, return a list of modules."
   (with-temp-buffer
     (let ((hoogle-error (call-process "hoogle" nil t nil "search" "--exact" ident)))
       (goto-char (point-min))
@@ -197,14 +214,22 @@ PROCESS."
                                     "\n"))))))
 
 (defun haskell-process-haskell-docs-ident (ident)
-  "Search with haskell-docs for IDENT, returns a list of modules."
-  (cl-remove-if-not (lambda (a) (string-match "^[A-Z][A-Za-b0-9_'.]+$" a))
-                    (split-string (shell-command-to-string (concat "haskell-docs --modules " ident))
-                                  "\n")))
+  "Search with haskell-docs for IDENT, return a list of modules."
+  (cl-remove-if-not
+   (lambda (a) (string-match "^[[:upper:]][[:alnum:]_'.]+$" a))
+   (split-string
+      (with-output-to-string
+        (with-current-buffer
+            standard-output
+          (call-process "haskell-docs"
+                        nil             ; no infile
+                        t               ; output to current buffer (that is string)
+                        nil             ; do not redisplay
+                        "--modules" ident)))
+      "\n")))
 
 (defun haskell-process-import-modules (process modules)
-  "Import `modules' with :m +, and send any import statements
-from `module-buffer'."
+  "Query PROCESS `:m +' command to import MODULES."
   (when haskell-process-auto-import-loaded-modules
     (haskell-process-queue-command
      process
@@ -217,14 +242,14 @@ from `module-buffer'."
 
 ;;;###autoload
 (defun haskell-describe (ident)
-  "Describe the given identifier."
+  "Describe the given identifier IDENT."
   (interactive (list (read-from-minibuffer "Describe identifier: "
                                            (haskell-ident-at-point))))
   (let ((results (read (shell-command-to-string
                         (concat "haskell-docs --sexp "
                                 ident)))))
     (help-setup-xref (list #'haskell-describe ident)
-		     (called-interactively-p 'interactive))
+                     (called-interactively-p 'interactive))
     (save-excursion
       (with-help-window (help-buffer)
         (with-current-buffer (help-buffer)
@@ -254,9 +279,10 @@ from `module-buffer'."
 
 ;;;###autoload
 (defun haskell-rgrep (&optional prompt)
-  "Grep the effective project for the symbol at point. Very
-useful for codebase navigation. Prompts for an arbitrary regexp
-given a prefix arg."
+  "Grep the effective project for the symbol at point.
+Very useful for codebase navigation.
+
+Prompts for an arbitrary regexp given a prefix arg PROMPT."
   (interactive "P")
   (let ((sym (if prompt
                  (read-from-minibuffer "Look for: ")
@@ -270,44 +296,63 @@ given a prefix arg."
   "Print info on the identifier at point.
 If PROMPT-VALUE is non-nil, request identifier via mini-buffer."
   (interactive "P")
-  (haskell-process-do-simple-echo
-   (let ((ident (replace-regexp-in-string
-                 "^!\\([A-Z_a-z]\\)"
-                 "\\1"
-                 (if prompt-value
-                     (read-from-minibuffer "Info: " (haskell-ident-at-point))
-                   (haskell-ident-at-point))))
-         (modname (unless prompt-value
-                    (haskell-utils-parse-import-statement-at-point))))
-     (if modname
-         (format ":browse! %s" modname)
-       (format (if (string-match "^[a-zA-Z_]" ident)
-                   ":info %s"
-                 ":info (%s)")
-               (or ident
-                   (haskell-ident-at-point)))))
-   'haskell-mode))
+  (let ((at-point (haskell-ident-at-point)))
+    (when (or prompt-value at-point)
+      (let* ((ident (replace-regexp-in-string
+                     "^!\\([A-Z_a-z]\\)"
+                     "\\1"
+                     (if prompt-value
+                         (read-from-minibuffer "Info: " at-point)
+                       at-point)))
+             (modname (unless prompt-value
+                        (haskell-utils-parse-import-statement-at-point)))
+             (command (cond
+                       (modname
+                        (format ":browse! %s" modname))
+                       ((string= ident "") ; For the minibuffer input case
+                        nil)
+                       (t (format (if (string-match "^[a-zA-Z_]" ident)
+                                      ":info %s"
+                                    ":info (%s)")
+                                  (or ident
+                                      at-point))))))
+        (when command
+          (haskell-process-show-repl-response command))))))
 
 ;;;###autoload
 (defun haskell-process-do-type (&optional insert-value)
-  "Print the type of the given expression."
+  ;; FIXME insert value functionallity seems to be missing.
+  "Print the type of the given expression.
+
+Given INSERT-VALUE prefix indicates that result type signature
+should be inserted."
   (interactive "P")
   (if insert-value
       (haskell-process-insert-type)
-    (haskell-process-do-simple-echo
-     (let ((ident (haskell-ident-at-point)))
-       ;; TODO: Generalize all these `string-match' of ident calls into
-       ;; one function.
-       (format (if (string-match "^[_[:lower:][:upper:]]" ident)
-                   ":type %s"
-                 ":type (%s)")
-               ident))
-     'haskell-mode)))
+    (let* ((expr
+            (if (use-region-p)
+                (buffer-substring-no-properties (region-beginning) (region-end))
+              (haskell-ident-at-point)))
+           (expr-okay (and expr
+                         (not (string-match-p "\\`[[:space:]]*\\'" expr))
+                         (not (string-match-p "\n" expr)))))
+      ;; No newlines in expressions, and surround with parens if it
+      ;; might be a slice expression
+      (when expr-okay
+        (haskell-process-show-repl-response
+         (format
+          (if (or (string-match-p "\\`(" expr)
+                  (string-match-p "\\`[_[:alpha:]]" expr))
+              ":type %s"
+            ":type (%s)")
+          expr))))))
 
 ;;;###autoload
 (defun haskell-mode-jump-to-def-or-tag (&optional next-p)
-  "Jump to the definition (by consulting GHCi), or (fallback)
-jump to the tag.
+  ;; FIXME NEXT-P arg is not used
+  "Jump to the definition.
+Jump to definition of identifier at point by consulting GHCi, or
+tag table as fallback.
 
 Remember: If GHCi is busy doing something, this will delay, but
 it will always be accurate, in contrast to tags, which always
@@ -322,24 +367,24 @@ position with `xref-pop-marker-stack'."
         (haskell-mode-handle-generic-loc loc)
       (call-interactively 'haskell-mode-tag-find))
     (unless (equal initial-loc (point-marker))
-      (save-excursion
-        (goto-char initial-loc)
-        (set-mark-command nil)
-        ;; Store position for return with `xref-pop-marker-stack'
-        (xref-push-marker-stack)))))
+      (with-current-buffer (marker-buffer initial-loc)
+        (save-excursion
+          (goto-char initial-loc)
+          (set-mark-command nil)
+          ;; Store position for return with `xref-pop-marker-stack'
+          (xref-push-marker-stack))))))
 
 ;;;###autoload
 (defun haskell-mode-goto-loc ()
-  "Go to the location of the thing at point. Requires the :loc-at
-command from GHCi."
+  "Go to the location of the thing at point.
+Requires the :loc-at command from GHCi."
   (interactive)
   (let ((loc (haskell-mode-loc-at)))
     (when loc
       (haskell-mode-goto-span loc))))
 
 (defun haskell-mode-goto-span (span)
-  "Jump to the span, whatever file and line and column it needs
-to to get there."
+  "Jump to the SPAN, whatever file and line and column it needs to get there."
   (xref-push-marker-stack)
   (find-file (expand-file-name (plist-get span :path)
                                (haskell-session-cabal-dir (haskell-interactive-session))))
@@ -348,81 +393,85 @@ to to get there."
   (forward-char (plist-get span :start-col)))
 
 (defun haskell-process-insert-type ()
-  "Get the identifer at the point and insert its type, if
-possible, using GHCi's :type."
-  (let ((process (haskell-interactive-process))
-        (query (let ((ident (haskell-ident-at-point)))
-                 (format (if (string-match "^[_[:lower:][:upper:]]" ident)
-                             ":type %s"
-                           ":type (%s)")
-                         ident))))
-    (haskell-process-queue-command
-     process
-     (make-haskell-command
-      :state (list process query (current-buffer))
-      :go (lambda (state)
-            (haskell-process-send-string (nth 0 state)
-                                         (nth 1 state)))
-      :complete (lambda (state response)
-                  (cond
-                   ;; TODO: Generalize this into a function.
-                   ((or (string-match "^Top level" response)
-                        (string-match "^<interactive>" response))
-                    (message response))
-                   (t
-                    (with-current-buffer (nth 2 state)
-                      (goto-char (line-beginning-position))
-                      (insert (format "%s\n" (replace-regexp-in-string "\n$" "" response)))))))))))
+  "Get the identifer at the point and insert its type.
+Use GHCi's :type if it's possible."
+  (let ((ident (haskell-ident-at-point)))
+    (when ident
+      (let ((process (haskell-interactive-process))
+            (query (format (if (string-match "^[_[:lower:][:upper:]]" ident)
+                               ":type %s"
+                             ":type (%s)")
+                           ident)))
+        (haskell-process-queue-command
+         process
+         (make-haskell-command
+          :state (list process query (current-buffer))
+          :go (lambda (state)
+                (haskell-process-send-string (nth 0 state)
+                                             (nth 1 state)))
+          :complete (lambda (state response)
+                      (cond
+                       ;; TODO: Generalize this into a function.
+                       ((or (string-match "^Top level" response)
+                            (string-match "^<interactive>" response))
+                        (message response))
+                       (t
+                        (with-current-buffer (nth 2 state)
+                          (goto-char (line-beginning-position))
+                          (insert (format "%s\n" (replace-regexp-in-string "\n$" "" response)))))))))))))
 
 (defun haskell-mode-find-def (ident)
-  "Find definition location of identifier. Uses the GHCi process
-to find the location.
+  ;; TODO Check if it possible to exploit `haskell-process-do-info'
+  "Find definition location of identifier IDENT.
+Uses the GHCi process to find the location.  Returns nil if it
+can't find the identifier or the identifier isn't a string.
 
 Returns:
 
     (library <package> <module>)
     (file <path> <line> <col>)
     (module <name>)
-"
-  (let ((reply (haskell-process-queue-sync-request
-                (haskell-interactive-process)
-                (format (if (string-match "^[a-zA-Z_]" ident)
-                            ":info %s"
-                          ":info (%s)")
-                        ident))))
-    (let ((match (string-match "-- Defined \\(at\\|in\\) \\(.+\\)$" reply)))
-      (when match
-        (let ((defined (match-string 2 reply)))
-          (let ((match (string-match "\\(.+?\\):\\([0-9]+\\):\\([0-9]+\\)$" defined)))
-            (cond
-             (match
-              (list 'file
-                    (expand-file-name (match-string 1 defined)
-                                      (haskell-session-current-dir (haskell-interactive-session)))
-                    (string-to-number (match-string 2 defined))
-                    (string-to-number (match-string 3 defined))))
-             (t
-              (let ((match (string-match "`\\(.+?\\):\\(.+?\\)'$" defined)))
-                (if match
-                    (list 'library
-                          (match-string 1 defined)
-                          (match-string 2 defined))
-                  (let ((match (string-match "`\\(.+?\\)'$" defined)))
-                    (if match
-                        (list 'module
-                              (match-string 1 defined))))))))))))))
+    nil"
+  (when (stringp ident)
+    (let ((reply (haskell-process-queue-sync-request
+                  (haskell-interactive-process)
+                  (format (if (string-match "^[a-zA-Z_]" ident)
+                              ":info %s"
+                            ":info (%s)")
+                          ident))))
+      (let ((match (string-match "-- Defined \\(at\\|in\\) \\(.+\\)$" reply)))
+        (when match
+          (let ((defined (match-string 2 reply)))
+            (let ((match (string-match "\\(.+?\\):\\([0-9]+\\):\\([0-9]+\\)$" defined)))
+              (cond
+               (match
+                (list 'file
+                      (expand-file-name (match-string 1 defined)
+                                        (haskell-session-current-dir (haskell-interactive-session)))
+                      (string-to-number (match-string 2 defined))
+                      (string-to-number (match-string 3 defined))))
+               (t
+                (let ((match (string-match "`\\(.+?\\):\\(.+?\\)'$" defined)))
+                  (if match
+                      (list 'library
+                            (match-string 1 defined)
+                            (match-string 2 defined))
+                    (let ((match (string-match "`\\(.+?\\)'$" defined)))
+                      (if match
+                          (list 'module
+                                (match-string 1 defined)))))))))))))))
 
 ;;;###autoload
 (defun haskell-mode-jump-to-def (ident)
-  "Jump to definition of identifier at point."
+  "Jump to definition of identifier IDENT at point."
   (interactive (list (haskell-ident-at-point)))
   (let ((loc (haskell-mode-find-def ident)))
     (when loc
       (haskell-mode-handle-generic-loc loc))))
 
 (defun haskell-mode-handle-generic-loc (loc)
-  "Either jump to or display a generic location. Either a file or
-a library."
+  "Either jump to or echo a generic location LOC.
+Either a file or a library."
   (cl-case (car loc)
     (file (haskell-mode-jump-to-loc (cdr loc)))
     (library (message "Defined in `%s' (%s)."
@@ -432,8 +481,8 @@ a library."
                      (elt loc 1)))))
 
 (defun haskell-mode-loc-at ()
-  "Get the location at point. Requires the :loc-at command from
-GHCi."
+  "Get the location at point.
+Requires the :loc-at command from GHCi."
   (let ((pos (or (when (region-active-p)
                    (cons (region-beginning)
                          (region-end)))
@@ -468,38 +517,9 @@ GHCi."
           (error (propertize "No reply. Is :loc-at supported?"
                              'face 'compilation-error)))))))
 
-(defun haskell-mode-type-at ()
-  "Get the type of the thing at point. Requires the :type-at
-command from GHCi."
-  (let ((pos (or (when (region-active-p)
-                   (cons (region-beginning)
-                         (region-end)))
-                 (haskell-spanable-pos-at-point)
-                 (cons (point)
-                       (point)))))
-    (when pos
-      (replace-regexp-in-string
-       "\n$"
-       ""
-       (save-excursion
-         (haskell-process-queue-sync-request
-          (haskell-interactive-process)
-          (replace-regexp-in-string
-           "\n"
-           " "
-           (format ":type-at %s %d %d %d %d %s"
-                   (buffer-file-name)
-                   (progn (goto-char (car pos))
-                          (line-number-at-pos))
-                   (1+ (current-column))
-                   (progn (goto-char (cdr pos))
-                          (line-number-at-pos))
-                   (1+ (current-column))
-                   (buffer-substring-no-properties (car pos)
-                                                   (cdr pos))))))))))
-
 ;;;###autoload
 (defun haskell-process-cd (&optional not-interactive)
+  ;; FIXME optional arg is not used
   "Change directory."
   (interactive)
   (let* ((session (haskell-interactive-session))
@@ -512,7 +532,10 @@ command from GHCi."
                                 dir)))
 
 (defun haskell-session-pwd (session &optional change)
-  "Prompt for the current directory."
+  "Prompt for the current directory.
+Return current working directory for SESSION.
+Optional CHANGE argument makes user to choose new working directory for SESSION.
+In this case new working directory path will be returned."
   (or (unless change
         (haskell-session-get session 'current-dir))
       (progn (haskell-session-set-current-dir
@@ -527,7 +550,8 @@ command from GHCi."
              (haskell-session-get session 'current-dir))))
 
 (defun haskell-process-change-dir (session process dir)
-  "Change the directory of the current process."
+  "Change SESSION's current directory.
+Query PROCESS to `:cd` to directory DIR."
   (haskell-process-queue-command
    process
    (make-haskell-command
@@ -552,7 +576,7 @@ command from GHCi."
                                          ":set -optP-include -optPdist/build/autogen/cabal_macros.h"))
 
 (defun haskell-process-do-try-info (sym)
-  "Get info of `sym' and echo in the minibuffer."
+  "Get info of SYM and echo in the minibuffer."
   (let ((process (haskell-interactive-process)))
     (haskell-process-queue-command
      process
@@ -570,7 +594,7 @@ command from GHCi."
                     (haskell-mode-message-line response)))))))
 
 (defun haskell-process-do-try-type (sym)
-  "Get type of `sym' and echo in the minibuffer."
+  "Get type of SYM and echo in the minibuffer."
   (let ((process (haskell-interactive-process)))
     (haskell-process-queue-command
      process
@@ -589,34 +613,95 @@ command from GHCi."
 
 ;;;###autoload
 (defun haskell-mode-show-type-at (&optional insert-value)
-  "Show the type of the thing at point."
+  "Show type of the thing at point or within active region asynchronously.
+This function requires GHCi-ng and `:set +c` option enabled by
+default (please follow GHCi-ng README available at URL
+`https://github.com/chrisdone/ghci-ng').
+
+\\<haskell-interactive-mode-map>
+To make this function works sometimes you need to load the file in REPL
+first using command `haskell-process-load-or-reload' bound to
+\\[haskell-process-load-or-reload].
+
+Optional argument INSERT-VALUE indicates that
+recieved type signature should be inserted (but only if nothing
+happened since function invocation)."
   (interactive "P")
-  (let ((ty (haskell-mode-type-at))
-        (orig (point)))
-    (if insert-value
-        (let ((ident-pos (haskell-ident-pos-at-point)))
-          (cond
-           ((region-active-p)
-            (delete-region (region-beginning)
-                           (region-end))
-            (insert "(" ty ")")
-            (goto-char (1+ orig)))
-           ((= (line-beginning-position) (car ident-pos))
-            (goto-char (line-beginning-position))
-            (insert (haskell-fontify-as-mode ty 'haskell-mode)
-                    "\n"))
-           (t
-            (save-excursion
-              (let ((col (save-excursion (goto-char (car ident-pos))
-                                         (current-column))))
-                (save-excursion (insert "\n")
-                                (indent-to col))
-                (insert (haskell-fontify-as-mode ty 'haskell-mode)))))))
-      (message "%s" (haskell-fontify-as-mode ty 'haskell-mode)))))
+  (let* ((pos (haskell-utils-capture-expr-bounds))
+         (req (haskell-utils-compose-type-at-command pos))
+         (process (haskell-interactive-process))
+         (buf (current-buffer))
+         (pos-reg (cons pos (region-active-p))))
+    (haskell-process-queue-command
+     process
+     (make-haskell-command
+      :state (list process req buf insert-value pos-reg)
+      :go
+      (lambda (state)
+        (let* ((prc (car state))
+               (req (nth 1 state)))
+          (haskell-utils-async-watch-changes)
+          (haskell-process-send-string prc req)))
+      :complete
+      (lambda (state response)
+        (let* ((init-buffer (nth 2 state))
+               (insert-value (nth 3 state))
+               (pos-reg (nth 4 state))
+               (wrap (cdr pos-reg))
+               (min-pos (caar pos-reg))
+               (max-pos (cdar pos-reg))
+               (sig (haskell-utils-reduce-string response))
+               (res-type (haskell-utils-parse-repl-response sig)))
+
+          (cl-case res-type
+            ;; neither popup presentation buffer
+            ;; nor insert response in error case
+            ('unknown-command
+             (message
+              (concat
+               "This command requires GHCi-ng. "
+               "Please read command description for details.")))
+            ('option-missing
+             (message
+              (concat
+               "Could not infer type signature. "
+               "You need to load file first. "
+               "Also :set +c is required. "
+               "Please read command description for details.")))
+            ('interactive-error (message "Wrong REPL response: %s" sig))
+            (otherwise
+             (if insert-value
+                 ;; Only insert type signature and do not present it
+                 (if (= (length haskell-utils-async-post-command-flag) 1)
+                     (if wrap
+                         ;; Handle region case
+                         (progn
+                           (deactivate-mark)
+                           (save-excursion
+                             (delete-region min-pos max-pos)
+                             (goto-char min-pos)
+                             (insert (concat "(" sig ")"))))
+                       ;; Non-region cases
+                       (haskell-utils-insert-type-signature sig))
+                   ;; Some commands registered, prevent insertion
+                   (let* ((rev (reverse haskell-utils-async-post-command-flag))
+                          (cs (format "%s" (cdr rev))))
+                     (message
+                      (concat
+                       "Type signature insertion was prevented. "
+                       "These commands were registered:"
+                       cs))))
+               ;; Present the result only when response is valid and not asked
+               ;; to insert result
+               (haskell-utils-echo-or-present response)))
+
+            (haskell-utils-async-stop-watching-changes init-buffer))))))))
 
 ;;;###autoload
 (defun haskell-process-generate-tags (&optional and-then-find-this-tag)
-  "Regenerate the TAGS table."
+  "Regenerate the TAGS table.
+If optional AND-THEN-FIND-THIS-TAG argument is present it is used with
+function `xref-find-definitions' after new table was generated."
   (interactive)
   (let ((process (haskell-interactive-process)))
     (haskell-process-queue-command
@@ -627,9 +712,9 @@ command from GHCi."
             (if (eq system-type 'windows-nt)
                 (haskell-process-send-string
                  (car state)
-                 (format ":!powershell -Command \"& { cd %s ; hasktags -e -x (ls -fi *.hs *.lhs *.hsc -exclude \\\"#*#\\\" -name -r) ; exit }\""
-                         (haskell-session-cabal-dir
-                          (haskell-process-session (car state)))))
+                 (format ":!hasktags --output=\"%s\\TAGS\" -x -e \"%s\""
+                            (haskell-session-cabal-dir (haskell-process-session (car state)))
+                            (haskell-session-cabal-dir (haskell-process-session (car state)))))
               (haskell-process-send-string
                (car state)
                (format ":!cd %s && %s | %s"
@@ -639,16 +724,19 @@ command from GHCi."
                        "xargs -0 hasktags -e -x"))))
       :complete (lambda (state response)
                   (when (cdr state)
-                    (let ((tags-file-name
-                           (haskell-session-tags-filename
-                            (haskell-process-session (car state)))))
-                      (find-tag (cdr state))))
+                    (let ((session-tags
+                          (haskell-session-tags-filename
+                           (haskell-process-session (car state)))))
+                      (add-to-list 'tags-table-list session-tags)
+                      (setq tags-file-name nil))
+                    (xref-find-definitions (cdr state)))
                   (haskell-mode-message-line "Tags generated."))))))
 
 (defun haskell-process-add-cabal-autogen ()
-  "Add <cabal-project-dir>/dist/build/autogen/ to the ghci search
-path. This allows modules such as 'Path_...', generated by cabal,
-to be loaded by ghci."
+  "Add cabal's autogen dir to the GHCi search path.
+Add <cabal-project-dir>/dist/build/autogen/ to GHCi seatch path.
+This allows modules such as 'Path_...', generated by cabal, to be
+loaded by GHCi."
   (unless (eq 'cabal-repl (haskell-process-type)) ;; redundant with "cabal repl"
     (let*
         ((session       (haskell-interactive-session))
@@ -660,8 +748,9 @@ to be loaded by ghci."
 
 ;;;###autoload
 (defun haskell-process-unignore ()
-  "Unignore any files that were specified as being ignored by the
-  inferior GHCi process."
+  "Unignore any ignored files.
+Do not ignore files that were specified as being ignored by the
+inferior GHCi process."
   (interactive)
   (let ((session (haskell-interactive-session))
         (changed nil))
@@ -686,7 +775,7 @@ to be loaded by ghci."
 
 ;;;###autoload
 (defun haskell-session-change-target (target)
-  "Set the build target for cabal repl"
+  "Set the build TARGET for cabal REPL."
   (interactive "sNew build target:")
   (let* ((session haskell-session)
          (old-target (haskell-session-get session 'target)))
@@ -708,9 +797,9 @@ to be loaded by ghci."
     (goto-char (+ column (point)))))
 
 (defun haskell-mode-buffer-apply-command (cmd)
-  "Execute shell command CMD with current buffer as input and
-replace the whole buffer with the output. If CMD fails the buffer
-remains unchanged."
+  "Execute shell command CMD with current buffer as input and output.
+Use buffer as input and replace the whole buffer with the
+output.  If CMD fails the buffer remains unchanged."
   (set-buffer-modified-p t)
   (let* ((chomp (lambda (str)
                   (while (string-match "\\`\n+\\|^\\s-+\\|\\s-+$\\|\n+\\'" str)
@@ -740,21 +829,23 @@ remains unchanged."
             (buffer-substring-no-properties (point-min) (point-max)))))
     (if (string= "" stderr-output)
         (if (string= "" stdout-output)
-            (funcall errout
-                     "Error: %s produced no output, leaving buffer alone" cmd)
+            (message "Error: %s produced no output, leaving buffer alone" cmd)
           (save-restriction
             (widen)
             ;; command successful, insert file with replacement to preserve
             ;; markers.
             (insert-file-contents tmp-file nil nil nil t)))
-      ;; non-null stderr, command must have failed
-      (funcall errout "%s failed: %s" cmd stderr-output))
+      (progn
+        ;; non-null stderr, command must have failed
+        (message "Error: %s ended with errors, leaving buffer alone" cmd)
+        ;; use (warning-minimum-level :debug) to see this
+        (display-warning cmd stderr-output :debug)))
     (delete-file tmp-file)
     (delete-file err-file)))
 
 ;;;###autoload
 (defun haskell-mode-find-uses ()
-  "Find uses of the identifier at point, highlight them all."
+  "Find use cases of the identifier at point and highlight them all."
   (interactive)
   (let ((spans (haskell-mode-uses-at)))
     (unless (null spans)
@@ -763,7 +854,7 @@ remains unchanged."
                do (haskell-mode-make-use-highlight span)))))
 
 (defun haskell-mode-make-use-highlight (span)
-  "Make a highlight overlay at the given span."
+  "Make a highlight overlay at the given SPAN."
   (save-window-excursion
     (save-excursion
       (haskell-mode-goto-span span)
@@ -781,8 +872,8 @@ remains unchanged."
            (point)))))))
 
 (defun haskell-mode-uses-at ()
-  "Get the locations of uses for the ident at point. Requires
-the :uses command from GHCi."
+  "Get the locations of use cases for the ident at point.
+Requires the :uses command from GHCi."
   (let ((pos (or (when (region-active-p)
                    (cons (region-beginning)
                          (region-end)))
@@ -822,4 +913,119 @@ the :uses command from GHCi."
           (error (propertize "No reply. Is :uses supported?"
                              'face 'compilation-error)))))))
 
+(defun haskell-utils-capture-expr-bounds ()
+  "Capture position bounds of expression at point.
+If there is an active region then it returns region
+bounds.  Otherwise it uses `haskell-spanable-pos-at-point` to
+capture identifier bounds.  If latter function returns NIL this function
+will return cons cell where min and max positions both are equal
+to point."
+  (or (when (region-active-p)
+        (cons (region-beginning)
+              (region-end)))
+      (haskell-spanable-pos-at-point)
+      (cons (point) (point))))
+
+(defun haskell-utils-compose-type-at-command (pos)
+  "Prepare :type-at command to be send to haskell process.
+POS is a cons cell containing min and max positions, i.e. target
+expression bounds."
+  (replace-regexp-in-string
+   "\n$"
+   ""
+   (format ":type-at %s %d %d %d %d %s"
+           (buffer-file-name)
+           (progn (goto-char (car pos))
+                  (line-number-at-pos))
+           (1+ (current-column))
+           (progn (goto-char (cdr pos))
+                  (line-number-at-pos))
+           (1+ (current-column))
+           (buffer-substring-no-properties (car pos)
+                                           (cdr pos)))))
+
+(defun haskell-utils-insert-type-signature (signature)
+  "Insert type signature.
+In case of active region is present, wrap it by parentheses and
+append SIGNATURE to original expression.  Otherwise tries to
+carefully insert SIGNATURE above identifier at point.  Removes
+newlines and extra whitespace in signature before insertion."
+  (let* ((ident-pos (or (haskell-ident-pos-at-point)
+                        (cons (point) (point))))
+         (min-pos (car ident-pos))
+         (sig (haskell-utils-reduce-string signature)))
+    (save-excursion
+      (goto-char min-pos)
+      (let ((col (current-column)))
+        (insert sig "\n")
+        (indent-to col)))))
+
+(defun haskell-utils-echo-or-present (msg)
+  "Present message in some manner depending on configuration.
+If variable `haskell-process-use-presentation-mode' is NIL it will output
+modified message MSG to echo area."
+  (if haskell-process-use-presentation-mode
+      (let ((session (haskell-process-session (haskell-interactive-process))))
+        (haskell-present session msg))
+    (let ((m (haskell-utils-reduce-string msg)))
+      (message m))))
+
+(defun haskell-utils-async-update-post-command-flag ()
+  "A special hook which collects triggered commands during async execution.
+This hook pushes value of variable `this-command' to flag variable
+`haskell-utils-async-post-command-flag'."
+  (let* ((cmd this-command)
+         (updated-flag (cons cmd haskell-utils-async-post-command-flag)))
+    (setq haskell-utils-async-post-command-flag updated-flag)))
+
+(defun haskell-utils-async-watch-changes ()
+  "Watch for triggered commands during async operation execution.
+Resets flag variable
+`haskell-utils-async-update-post-command-flag' to NIL.  By chanhges it is
+assumed that nothing happened, e.g. nothing was inserted in
+buffer, point was not moved, etc.  To collect data `post-command-hook' is used."
+  (setq haskell-utils-async-post-command-flag nil)
+  (add-hook
+   'post-command-hook #'haskell-utils-async-update-post-command-flag nil t))
+
+(defun haskell-utils-async-stop-watching-changes (buffer)
+  "Clean up after async operation finished.
+This function takes care about cleaning up things made by
+`haskell-utils-async-watch-changes'.  The BUFFER argument is a buffer where
+`post-command-hook' should be disabled.  This is neccessary, because
+it is possible that user will change buffer during async function
+execusion."
+  (with-current-buffer buffer
+    (setq haskell-utils-async-post-command-flag nil)
+    (remove-hook
+     'post-command-hook #'haskell-utils-async-update-post-command-flag t)))
+
+(defun haskell-utils-reduce-string (s)
+  "Remove newlines ans extra whitespace from S.
+Removes all extra whitespace at the beginning of each line leaving
+only single one.  Then removes all newlines."
+  (let ((s_ (replace-regexp-in-string "^\s+" " " s)))
+    (replace-regexp-in-string "\n" "" s_)))
+
+(defun haskell-utils-parse-repl-response (r)
+  "Parse response R from REPL and return special kind of result.
+The result is response string itself with speacial property
+response-type added.
+
+This property could be of the following:
+
++ unknown-command
++ option-missing
++ interactive-error
++ success"
+  (let ((first-line (car (split-string r "\n"))))
+    (cond
+     ((string-match-p "^unknown command" first-line) 'unknown-command)
+     ((string-match-p "^Couldn't guess that module name. Does it exist?"
+                      first-line)
+      'option-missing)
+     ((string-match-p "^<interactive>:" first-line) 'interactive-error)
+     (t 'success))))
+
 (provide 'haskell-commands)
+;;; haskell-commands.el ends here
