@@ -27,10 +27,11 @@
 
 ;;; Code:
 
+(require 'cl-lib)
 (require 'haskell-compat)
 (require 'haskell-session)
 (require 'haskell-process)
-(with-no-warnings (require 'cl))
+(require 'haskell-interactive-mode)
 
 (defcustom haskell-menu-buffer-name "*haskell-menu*"
   "The name of the Haskell session menu buffer"
@@ -44,7 +45,8 @@
   (or (get-buffer haskell-menu-buffer-name)
       (with-current-buffer (get-buffer-create haskell-menu-buffer-name)
         (haskell-menu-mode)))
-  (switch-to-buffer-other-window (get-buffer haskell-menu-buffer-name)))
+  (switch-to-buffer-other-window (get-buffer haskell-menu-buffer-name))
+  (haskell-menu-revert-function nil nil))
 
 (define-derived-mode haskell-menu-mode special-mode "Haskell Session Menu"
   "Major mode for managing Haskell sessions.
@@ -56,14 +58,10 @@ Letters do not insert themselves; instead, they are commands."
   (setq truncate-lines t)
   (haskell-menu-revert-function nil t))
 
-(defvar haskell-menu-mode-map
-  (let ((map (make-keymap)))
-    (suppress-keymap map t)
-    (define-key map (kbd "n") 'next-line)
-    (define-key map (kbd "p") 'previous-line)
-    (define-key map (kbd "RET") 'haskell-menu-mode-ret)
-    map)
-  "Key map for haskell-menu-mode.")
+(suppress-keymap haskell-menu-mode-map t)
+(define-key haskell-menu-mode-map (kbd "n") 'next-line)
+(define-key haskell-menu-mode-map (kbd "p") 'previous-line)
+(define-key haskell-menu-mode-map (kbd "RET") 'haskell-menu-mode-ret)
 
 (defun haskell-menu-revert-function (arg1 arg2)
   "Function to refresh the display."
@@ -93,7 +91,7 @@ Letters do not insert themselves; instead, they are commands."
                            (if (process-live-p process) (number-to-string id) "-")
                            (if (process-live-p process)
                                (format-time-string "%H:%M:%S"
-                                                   (encode-time (caddr (assoc 'etime (process-attributes id)))
+                                                   (encode-time (cl-caddr (assoc 'etime (process-attributes id)))
                                                                 0 0 0 0 0))
                              "-")
                            (if (process-live-p process)
@@ -149,16 +147,12 @@ Letters do not insert themselves; instead, they are commands."
                                                  (progn (search-forward " ")
                                                         (forward-char -1)
                                                         (point)))))
-         (session (car (remove-if-not (lambda (session)
-                                        (string= (haskell-session-name session)
-                                                 name))
-                                      haskell-sessions))))
+         (session (car (cl-remove-if-not (lambda (session)
+                                           (string= (haskell-session-name session)
+                                                    name))
+                                         haskell-sessions))))
     (switch-to-buffer (haskell-session-interactive-buffer session))))
 
 (provide 'haskell-menu)
 
-;; Local Variables:
-;; byte-compile-warnings: (not cl-functions)
-;; End:
-
 ;;; haskell-menu.el ends here
