@@ -9,7 +9,7 @@
 ;;          2001-2002 Reuben Thomas (>=v1.4)
 ;;          2003      Dave Love <fx@gnu.org>
 ;; Keywords: faces files Haskell
-;; Version: 13.12
+;; Version: 13.14.2
 ;; URL: https://github.com/haskell/haskell-mode
 
 ;; This file is not part of GNU Emacs.
@@ -142,24 +142,18 @@
 ;; All functions/variables start with `(literate-)haskell-'.
 
 ;; Version of mode.
-(defconst haskell-version "@VERSION@"
+(defconst haskell-version "13.14"
   "The release version of `haskell-mode'.")
 
-(defconst haskell-git-version "@GIT_VERSION@"
-  "The Git version of `haskell-mode'.")
-
 ;;;###autoload
 (defun haskell-version (&optional here)
   "Show the `haskell-mode` version in the echo area.
-With prefix argument HERE, insert it at point.
-When FULL is non-nil, use a verbose version string.
-When MESSAGE is non-nil, display a message with the version."
+With prefix argument HERE, insert it at point."
   (interactive "P")
   (let* ((haskell-mode-dir (ignore-errors
                              (file-name-directory (or (locate-library "haskell-mode") ""))))
-         (_version (format "haskell-mode version %s (%s @ %s)"
+         (_version (format "haskell-mode version %s (%s)"
                            haskell-version
-                           haskell-git-version
                            haskell-mode-dir)))
     (if here
         (insert _version)
@@ -194,7 +188,6 @@ be set to the preferred literate style."
   :group 'haskell
   :type '(choice (const bird) (const tex) (const nil)))
 
-;;;###autoload
 (defvar haskell-mode-map
   (let ((map (make-sparse-keymap)))
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -212,9 +205,7 @@ be set to the preferred literate style."
 (defun haskell-mode-enable-process-minor-mode ()
   "Tell the user to choose a minor mode for process interaction."
   (interactive)
-  (error "You tried to do an interaction command, but an interaction mode has not been enabled yet.
-
-Run M-x describe-variable haskell-mode-hook for a list of such modes."))
+  (error "Run `C-h f haskell-mode` for instruction how to setup a Haskell interaction mode."))
 
 (easy-menu-define haskell-mode-menu haskell-mode-map
   "Menu for the Haskell major mode."
@@ -459,7 +450,7 @@ Run M-x describe-variable haskell-mode-hook for a list of such modes."))
     (modify-syntax-entry ?\t " " table)
     (modify-syntax-entry ?\" "\"" table)
     (modify-syntax-entry ?\' "_" table)
-    (modify-syntax-entry ?_  "_" table)
+    (modify-syntax-entry ?_  "w" table)
     (modify-syntax-entry ?\( "()" table)
     (modify-syntax-entry ?\) ")(" table)
     (modify-syntax-entry ?\[  "(]" table)
@@ -471,10 +462,10 @@ Run M-x describe-variable haskell-mode-hook for a list of such modes."))
     (modify-syntax-entry ?\n ">" table)
 
     (modify-syntax-entry ?\` "$`" table)
-    (modify-syntax-entry ?\\ "\\" table)
+
     (mapc (lambda (x)
             (modify-syntax-entry x "." table))
-          "!#$%&*+./:<=>?@^|~")
+          "!#$%&*+./:<=>?@^|~,;\\")
 
     ;; Haskell symbol characters are treated as punctuation because
     ;; they are not able to form identifiers with word constituent 'w'
@@ -493,19 +484,18 @@ Run M-x describe-variable haskell-mode-hook for a list of such modes."))
 May return a qualified name."
   (let ((reg (haskell-ident-pos-at-point)))
     (when reg
-      (unless (= (car reg) (cdr reg))
-        (buffer-substring-no-properties (car reg) (cdr reg))))))
+      (buffer-substring-no-properties (car reg) (cdr reg)))))
 
 (defun haskell-spanable-pos-at-point ()
   "Like `haskell-ident-pos-at-point', but includes any surrounding backticks."
   (save-excursion
     (let ((pos (haskell-ident-pos-at-point)))
-      (if pos
-          (cl-destructuring-bind (start . end) pos
-            (if (and (eq ?` (char-before start))
-                     (eq ?` (char-after end)))
-                (cons (- start 1) (+ end 1))
-              (cons start end)))))))
+      (when pos
+        (cl-destructuring-bind (start . end) pos
+          (if (and (eq ?` (char-before start))
+                   (eq ?` (char-after end)))
+              (cons (- start 1) (+ end 1))
+            (cons start end)))))))
 
 (defun haskell-ident-pos-at-point ()
   "Return the span of the identifier under point, or nil if none found.
@@ -543,7 +533,8 @@ May return a qualified name."
                     (looking-at "[[:upper:]]"))
           (setq start (point)))
         ;; This is it.
-        (cons start end)))))
+        (unless (= start end)
+          (cons start end))))))
 
 (defun haskell-delete-indentation (&optional arg)
   "Like `delete-indentation' but ignoring Bird-style \">\"."
@@ -559,70 +550,6 @@ May return a qualified name."
   :type 'boolean
   :group 'haskell-interactive)
 
-(defcustom haskell-mode-hook nil
-  "Hook run after entering `haskell-mode'.
-
-You may be looking at this documentation because you haven't
-configured indentation or process interaction.
-
-Indentation modes:
-
-   `haskell-indentation-mode', Kristof Bastiaensen
-     Intelligent semi-automatic indentation Mk2
-
-   `haskell-indent-mode', Guy Lapalme
-     Intelligent semi-automatic indentation.
-
-   `haskell-simple-indent-mode', Graeme E Moss and Heribert Schuetz
-     Simple indentation.
-
-Interaction modes:
-
-   `interactive-haskell-mode'
-     Interact with per-project GHCi processes through a REPL and
-     directory-aware sessions.
-
-   `inf-haskell-mode'
-     Interact with a GHCi process using comint-mode. Deprecated.
-
-Other modes:
-
-   `haskell-decl-scan-mode', Graeme E Moss
-     Scans top-level declarations, and places them in a menu.
-
-   `haskell-doc-mode', Hans-Wolfgang Loidl
-     Echoes types of functions or syntax of keywords when the cursor is idle.
-
-To activate a minor-mode, simply run the interactive command. For
-example, `M-x haskell-doc-mode'. Run it again to disable it.
-
-To enable a mode for every haskell-mode buffer, add a hook in
-your Emacs configuration. For example, to enable
-haskell-indent-mode and interactive-haskell-mode, use the
-following:
-
-(add-hook 'haskell-mode-hook 'haskell-indent-mode)
-(add-hook 'haskell-mode-hook 'interactive-haskell-mode)
-
-See Info node `(haskell-mode)haskell-mode-hook' for more details.
-
-Warning: do not enable more than one of the three indentation
-modes. See Info node `(haskell-mode)indentation' for more
-details."
-  :type 'hook
-  :group 'haskell
-  :link '(info-link "(haskell-mode)haskell-mode-hook")
-  :link '(function-link haskell-mode)
-  :options `(capitalized-words-mode
-             imenu-add-menubar-index
-             turn-on-eldoc-mode
-             turn-on-haskell-decl-scan
-             turn-on-haskell-doc
-             turn-on-haskell-indent
-             turn-on-haskell-indentation
-             turn-on-haskell-simple-indent
-             turn-on-haskell-unicode-input-method))
-
 (defvar eldoc-print-current-symbol-info-function)
 
 ;; For compatibility with Emacs < 24, derive conditionally
@@ -634,19 +561,69 @@ details."
 (define-derived-mode haskell-mode haskell-parent-mode "Haskell"
   "Major mode for editing Haskell programs.
 
-See also Info node `(haskell-mode)Getting Started' for more
-information about this mode.
+For more information aee also Info node `(haskell-mode)Getting Started'.
 
 \\<haskell-mode-map>
-Literate scripts are supported via `literate-haskell-mode'.
+
+Literate Haskell scripts are supported via `literate-haskell-mode'.
 The variable `haskell-literate' indicates the style of the script in the
 current buffer.  See the documentation on this variable for more details.
 
 Use `haskell-version' to find out what version of Haskell mode you are
 currently using.
 
-Additional Haskell mode modules can be hooked in via `haskell-mode-hook';
-see documentation for that variable for more details."
+Additional Haskell mode modules can be hooked in via `haskell-mode-hook'.
+
+Indentation modes:
+
+    `haskell-indentation-mode', Kristof Bastiaensen, Gergely Risko
+      Intelligent semi-automatic indentation Mk2
+
+    `haskell-indent-mode', Guy Lapalme
+      Intelligent semi-automatic indentation.
+
+    `haskell-simple-indent-mode', Graeme E Moss and Heribert Schuetz
+      Simple indentation.
+
+Interaction modes:
+
+    `interactive-haskell-mode'
+      Interact with per-project GHCi processes through a REPL and
+      directory-aware sessions.
+
+    `inf-haskell-mode'
+      Interact with a GHCi process using comint-mode. Deprecated.
+
+Other modes:
+
+    `haskell-decl-scan-mode', Graeme E Moss
+      Scans top-level declarations, and places them in a menu.
+
+    `haskell-doc-mode', Hans-Wolfgang Loidl
+      Echoes types of functions or syntax of keywords when the cursor is idle.
+
+To activate a minor-mode, simply run the interactive command. For
+example, `M-x haskell-doc-mode'. Run it again to disable it.
+
+To enable a mode for every haskell-mode buffer, add a hook in
+your Emacs configuration. To do that you can customize
+`haskell-mode-hook' or add lines to your .emacs file. For
+example, to enable `haskell-indent-mode' and
+`interactive-haskell-mode', use the following:
+
+    (add-hook 'haskell-mode-hook 'haskell-indentation-mode)
+    (add-hook 'haskell-mode-hook 'interactive-haskell-mode)
+
+For more details see Info node `(haskell-mode)haskell-mode-hook'.
+
+Warning: do not enable more than one of the above indentation
+modes. See Info node `(haskell-mode)indentation' for more
+details.
+
+Minor modes that work well with `haskell-mode':
+
+- `smerge-mode': show and work with diff3 conflict markers used
+  by git, svn and other version control systems."
   :group 'haskell
   ;; paragraph-{start,separate} should treat comments as paragraphs as well.
   (set (make-local-variable 'paragraph-start)
@@ -781,6 +758,8 @@ see documentation for that variable for more details."
 ;;;###autoload
 (add-to-list 'auto-mode-alist        '("\\.l[gh]s\\'" . literate-haskell-mode))
 ;;;###autoload
+(add-to-list 'auto-mode-alist        '("\\.hsc\\'" . haskell-mode))
+;;;###autoload
 (add-to-list 'interpreter-mode-alist '("runghc" . haskell-mode))
 ;;;###autoload
 (add-to-list 'interpreter-mode-alist '("runhaskell" . haskell-mode))
@@ -795,6 +774,15 @@ If nil, use the Hoogle web-site."
   :type '(choice (const :tag "Use Web-site" nil)
                  string))
 
+(defcustom haskell-hoogle-url "http://haskell.org/hoogle/?q=%s"
+  "Default value for hoogle web site.
+"
+  :group 'haskell
+  :type '(choice
+          (const :tag "haskell-org" "http://haskell.org/hoogle/?q=%s")
+          (const :tag "fp-complete" "https://www.fpcomplete.com/hoogle?q=%s")
+          string))
+
 ;;;###autoload
 (defun haskell-hoogle (query &optional info)
   "Do a Hoogle search for QUERY.
@@ -812,7 +800,7 @@ is asked to show extra info for the items matching QUERY.."
                         nil nil def)
            current-prefix-arg)))
   (if (null haskell-hoogle-command)
-      (browse-url (format "http://haskell.org/hoogle/?q=%s" query))
+      (browse-url (format haskell-hoogle-url (url-hexify-string query)))
     (let ((hoogle-args (append (when info '("-i"))
                                (list "--color" (shell-quote-argument query)))))
       (with-help-window "*hoogle*"
@@ -865,6 +853,14 @@ is asked to show extra info for the items matching QUERY.."
           (hoogle-start-server)
         (error "hoogle is not installed")))))
 
+(defcustom haskell-hayoo-url "http://hayoo.fh-wedel.de/?query=%s"
+  "Default value for hayoo web site.
+"
+  :group 'haskell
+  :type '(choice
+          (const :tag "fh-wedel.de" "http://hayoo.fh-wedel.de/?query=%s")
+          string))
+
 ;;;###autoload
 (defun haskell-hayoo (query)
   "Do a Hayoo search for QUERY."
@@ -875,7 +871,7 @@ is asked to show extra info for the items matching QUERY.."
                             (format "Hayoo query (default %s): " def)
                           "Hayoo query: ")
                         nil nil def))))
-  (browse-url (format "http://holumbus.fh-wedel.de/hayoo/hayoo.html?query=%s" query)))
+  (browse-url (format haskell-hayoo-url (url-hexify-string query))))
 
 ;;;###autoload
 (defalias 'hayoo 'haskell-hayoo)
