@@ -27,6 +27,7 @@
 (require 'haskell-commands)
 (require 'haskell-sandbox)
 (require 'haskell-modules)
+(require 'haskell-string)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Basic configuration hooks
@@ -58,7 +59,7 @@
 (defun haskell-process-completions-at-point ()
   "A completion-at-point function using the current haskell process."
   (when (haskell-session-maybe)
-    (let ((process (haskell-process)) symbol)
+    (let ((process (haskell-process)) symbol symbol-bounds)
       (cond
        ;; ghci can complete module names, but it needs the "import "
        ;; string at the beginning
@@ -66,17 +67,37 @@
                           "import" (1+ space)
                           (? "qualified" (1+ space))
                           (group (? (char upper) ; modid
-                                    (* (char alnum ?' ?.))))))
+                                    (* (char alnum ?' ?.)))))
+                      (line-beginning-position))
         (let ((text (match-string-no-properties 0))
               (start (match-beginning 1))
               (end (match-end 1)))
           (list start end
                 (haskell-process-get-repl-completions process text))))
-       ((setq symbol (symbol-at-point))
-        (cl-destructuring-bind (start . end) (bounds-of-thing-at-point 'symbol)
-          (let ((completions (haskell-process-get-repl-completions
-                              process (symbol-name symbol))))
-            (list start end completions))))))))
+       ;; Complete OPTIONS using :complete repl ":set ..."
+       ((and (nth 4 (syntax-ppss))
+           (save-excursion
+             (let ((p (point)))
+               (and (search-backward "{-#" nil t)
+                  (search-forward-regexp "\\_<OPTIONS\\(?:_GHC\\)?\\_>" p t))))
+           (looking-back
+            (rx symbol-start "-" (* (char alnum ?-)))
+            (line-beginning-position)))
+        (list (match-beginning 0) (match-end 0) haskell-ghc-supported-options))
+       ;; Complete LANGUAGE :complete repl ":set -X..."
+       ((and (nth 4 (syntax-ppss))
+           (save-excursion
+             (let ((p (point)))
+               (and (search-backward "{-#" nil t)
+                  (search-forward-regexp "\\_<LANGUAGE\\_>" p t))))
+           (setq symbol-bounds (bounds-of-thing-at-point 'symbol)))
+        (list (car symbol-bounds) (cdr symbol-bounds)
+              haskell-ghc-supported-extensions))
+       ((setq symbol-bounds (haskell-ident-pos-at-point))
+        (cl-destructuring-bind (start . end) symbol-bounds
+          (list start end
+                (haskell-process-get-repl-completions
+                 process (buffer-substring-no-properties start end)))))))))
 
 ;;;###autoload
 (defun haskell-interactive-mode-return ()
@@ -189,18 +210,39 @@
   "Prompt to restart the died process."
   (let ((process-name (haskell-process-name process)))
     (if haskell-process-suggest-restart
-        (cl-case (read-event
-                  (propertize (format "The Haskell process `%s' has died. Restart? (y, n, l: show process log)"
-                                      process-name)
-                              'face 'minibuffer-prompt))
-          (?y (haskell-process-start (haskell-process-session process)))
-          (?l (let* ((response (haskell-process-response process))
-                     (buffer (get-buffer "*haskell-process-log*")))
-                (if buffer
-                    (switch-to-buffer buffer)
-                  (progn (switch-to-buffer (get-buffer-create "*haskell-process-log*"))
-                         (insert response)))))
-          (?n))
+        (cond
+         ((string-match "You need to re-run the 'configure' command."
+                        (haskell-process-response process))
+          (cl-case (read-event
+                    (concat "The Haskell process ended. Cabal wants you to run "
+                            (propertize "cabal configure" 'face 'font-lock-keyword-face)
+                            " because there is a version mismatch. Re-configure (y, n, l: view log)?"
+                            "\n\n"
+                            "Cabal said:\n\n"
+                            (propertize (haskell-process-response process)
+                                        'face 'font-lock-comment-face)))
+            (?y (let ((default-directory (haskell-session-cabal-dir (haskell-process-session process))))
+                  (message "%s" (shell-command-to-string "cabal configure"))))
+            (?l (let* ((response (haskell-process-response process))
+                       (buffer (get-buffer "*haskell-process-log*")))
+                  (if buffer
+                      (switch-to-buffer buffer)
+                    (progn (switch-to-buffer (get-buffer-create "*haskell-process-log*"))
+                           (insert response)))))
+            (?n)))
+         (t
+          (cl-case (read-event
+                    (propertize (format "The Haskell process `%s' has died. Restart? (y, n, l: show process log)"
+                                        process-name)
+                                'face 'minibuffer-prompt))
+            (?y (haskell-process-start (haskell-process-session process)))
+            (?l (let* ((response (haskell-process-response process))
+                       (buffer (get-buffer "*haskell-process-log*")))
+                  (if buffer
+                      (switch-to-buffer buffer)
+                    (progn (switch-to-buffer (get-buffer-create "*haskell-process-log*"))
+                           (insert response)))))
+            (?n))))
       (message (format "The Haskell process `%s' is dearly departed."
                        process-name)))))
 
@@ -257,11 +299,11 @@
                           (insert (cdr mapping)))
                  (insert module)))
              (haskell-mode-format-imports)))
-          ((not (string= "" (save-excursion (forward-char -1) (haskell-ident-at-point))))
+          (t
            (let ((ident (save-excursion (forward-char -1) (haskell-ident-at-point))))
              (insert " ")
-             (haskell-process-do-try-info ident)))
-          (t (insert " ")))))
+             (when ident
+               (haskell-process-do-try-info ident)))))))
 
 ;;;###autoload
 (defun haskell-mode-jump-to-tag (&optional next-p)
@@ -270,9 +312,10 @@
   (let ((ident (haskell-ident-at-point))
         (tags-file-name (haskell-session-tags-filename (haskell-session)))
         (tags-revert-without-query t))
-    (when (not (string= "" (haskell-trim ident)))
+    (when (and ident (not (string= "" (haskell-string-trim ident))))
       (cond ((file-exists-p tags-file-name)
-             (find-tag ident next-p))
+             (let ((xref-prompt-for-identifier next-p))
+               (xref-find-definitions ident)))
             (t (haskell-process-generate-tags ident))))))
 
 ;;;###autoload
